/*
 * skillchains-windows.js
 *
 * Copyright (c) 2004 by Daniel Potter.  All rights reserved.
 *
 * This file is part of the FFXI Skillchains Tool.
 *
 * The FFXI Skillchains Tool is free software; you can redistribute
 * it and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * The FFXI Skillchains Tool is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the FFXI Skillchains Tool; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 *
 * Handles displaying of the various windows.
 */

// Do the upgrade check - in a block so this will get GCed immediately
// after the window opens.
{
    // If this was an upgrade and old data is currently stored in the prefs
    // or persisted window data, remove it.  Grab the profile branch for
    // skillchain information.
    var prefs = Components.classes["@mozilla.org/preferences-service;1"].
               getService(Components.interfaces.nsIPrefService);
    prefs = prefs.getBranch("skillchains.");
    var thisVersion = "0.8.0";
    // See if we have a last version tag
    var version = null;
    try {
	version = prefs.getCharPref("last.version");
    } catch (ex) {
	// No, we do not - either new install or upgrade from a version
	// that didn't set a last version indicator
    }
    try {
	if (prefs.getBoolPref("clock")) {
	    // Display a warning message about the clock being removed.
	    // Note that this will (thankfully) not happen multiple times
	    // because we kill this pref below
	    // Don't bother localizing this message, because it was never
	    // available in another language anyway.
	    addEventListener("load", function(event) {
		if (confirm("The Vana'diel Clock has been removed from this version of the\nFFXI Skillchains Calculator.\n\nIt can be installed as a separate extension now.\n\nTo visit the new Vana'diel Clock extension website, where you\ncan download the new extension, click OK.")) {
		    window.open("http://ffxi.xenoveritas.org/tools/vanaclock/");
		}
	    }, false);
	    prefs.clearUserPref("clock");
	}
    } catch (ex) {
	// Don't care if this isn't set
    }
    if (version == null || version != thisVersion) {
	function removePref(prefs, pref) {
	    try {
		prefs.clearUserPref(pref);
		dump("Removed " + pref + "\n");
	    } catch (ex) { }
	}
	function removeProperty(rdf, ds, obj, prop) {
	    try {
		var property = rdf.GetResource(prop);
	    } catch (ex) {
		dump("Can't create resource for property \"" + prop + "\"\n");
		return;
	    }
	    try {
		var target = ds.GetTarget(obj, property, true);
		if (target == null)
		    return;
	    } catch (ex) {
		dump("Can't get target for " + prop + "\n");
		return;
	    }
	    try {
		ds.Unassert(obj, property, target);
	    } catch (ex) {
		dump("Can't unassert " + prop + "\n");
	    }
	}
	try {
	    // Upgrade to this version now
	    dump("=== Upgrading to Skillchain Extension " + thisVersion);
	    if (version != null)
		dump(" from " + version);
	    dump(" ===\n");
	    // Remove old preferences, if they exist
	    removePref(prefs, "clock");
	    removePref(prefs, "moon");
	    removePref(prefs, "clock.date");
	    // Next remove persisted data that should no longer be persisted
	    var rdfService = Components.classes["@mozilla.org/rdf/rdf-service;1"].
                   getService(Components.interfaces.nsIRDFService);
	    var localStore = rdfService.GetDataSource("rdf:local-store");
	    var aboutXULResource = rdfService.GetResource("chrome://skillchains/content/about.xul");
	    var persistResource = rdfService.GetResource("http://home.netscape.com/NC-rdf#persist");
	    var aboutResource = rdfService.GetResource("chrome://skillchains/content/about.xul#skillchains-about-window");
	    removeProperty(rdfService, localStore, aboutResource, "screenX");
	    removeProperty(rdfService, localStore, aboutResource, "screenY");
	    var target = localStore.GetTarget(aboutXULResource, persistResource, true);
	    if (target == null) {
		// Cool, it's already dead.
	    } else {
		try {
		    localStore.Unassert(aboutXULResource, persistResource, target);
		} catch (ex) {
		    dump("Unable to remove assertion for chrome://skillchains/content/about.xul => http://home.netscape.com/NC-rdf#persist\n");
		    dump(ex);
		    dump("\n");
		}
	    }
	    var optionsXULResource = rdfService.GetResource("chrome://skillchains/content/skillchains-options.xul");
	    var optionsResource = rdfService.GetResource("chrome://skillchains/content/skillchains-options.xul#skillchains-options-window");
	    removeProperty(rdfService, localStore, optionsResource, "screenX");
	    removeProperty(rdfService, localStore, optionsResource, "screenY");
	    target = localStore.GetTarget(optionsXULResource, persistResource, true);
	    if (target == null) {
		// Cool, it's already dead.
	    } else {
		try {
		    localStore.Unassert(optionsXULResource, persistResource, target);
		} catch (ex) {
		    dump("Unable to remove assertion for chrome://skillchains/content/about.xul => http://home.netscape.com/NC-rdf#persist\n");
		    dump(ex);
		    dump("\n");
		}
	    }
	    // Finally, set the version indicator
	    dump("=== Upgrade to Skillchain Extension " + thisVersion + " Completed Successfully ===\n");
	    prefs.setCharPref("last.version", thisVersion);
	} catch (ex) {
	    dump("Unable to upgrade the Skillchains Extension to " + thisVersion + ":\n" + ex);
	}
    }
}

function extension_showSkillchainsTool() {
    var win = window.open("chrome://skillchains/content/skillchains.xul", "extension_skillchains", "chrome, resizable=yes");
    if (win) {
	win.focus();
    }
}

/*
 * Pops up the database of skillchains for individual weapons.
 */
function extension_showSkillchainsDatabase() {
    var win = window.open("chrome://skillchains/content/skillchains-database.xul", "extension_skillchains_database", "chrome, resizable=yes");
    if (win) {
	win.focus();
    }
}

function extension_showSkillchainsUsage() {
    window.open("http://ffxi.xenoveritas.org/tools/skillchains/usage.shtml");
}

function extension_showSkillchainsAbout() {
    var win = window.openDialog("chrome://skillchains/content/about.xul", "extension_skillchains_about", "chrome");
    if (win) {
	win.focus();
    }
}

function extension_showSkillchainsOptions() {
    var win = window.openDialog("chrome://skillchains/content/skillchains-options.xul", "extension_skillchains_options", "chrome,dependent");
    if (win) {
	win.focus();
    }
}