/*
 * skillchains-xul.js
 *
 * Copyright (c) 2004 by Daniel Potter.  All rights reserved.
 *
 * This file is part of the FFXI Skillchains Tool.
 *
 * The FFXI Skillchains Tool is free software; you can redistribute
 * it and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * The FFXI Skillchains Tool is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the FFXI Skillchains Tool; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 *
 * Controls the XUL UI.
 */

// The number of party members we display and allow
// (some future version may allow more than this, but really, there's practically
// no point in going over 6)
const MAX_PARTY_MEMBERS = 6;

// Namespace for HTML, used when generating the new output.
const HTML_NS = "";

var currentProgressCount = 0;
var totalProgressCount = 1;
var progressInterval = null;

var elementGUIDCounter = 0;

function init() {
    var rows = document.getElementById("skill-rows");
    var row, label, textbox;
    var nonelabel = "(" + getLocalString("none") + ")";
    var anylabel = "(" + getLocalString("any") + ")";
    /*var exlabel = getLocalString("ex");
    var splabel = getLocalString("sp");*/
    // Find our stylesheet
    for (var i = 0; i < document.styleSheets.length; i++) {
	if (document.styleSheets[i].href == "chrome://skillchains/skin/skillchains.css") {
	    insertColorStyleRules(document.styleSheets[i]);
	    break;
	}
    }
    for (var i = 1; i <= MAX_PARTY_MEMBERS; i++) {
	row = document.createElement("row");
	row.setAttribute("align", "center");
	row.appendChild(createMemberTextbox(i));
	row.appendChild(createWeaponDropdown(i, nonelabel));
	row.appendChild(createSkillBox(i));
	row.appendChild(createJobDropdown(i, "job", nonelabel));
	// And again for the support job
	row.appendChild(createJobDropdown(i, "subjob", anylabel));
	row.appendChild(createLabel("member" + i + "-ex"));
	row.appendChild(createLabel("member" + i + "-sp"));
	row.appendChild(createColorDropdown(i));
	rows.appendChild(row);
    }
    // One last "row" to allow setting all skills at once
    row = document.createElement("row");
    row.setAttribute("align", "center");
    label = document.createElement("label");
    label.setAttribute("value", getLocalString("all") + ":");
    label.setAttribute("align", "end");
    row.appendChild(label);
    row.appendChild(document.createElement("spacer"));
    textbox = document.createElement("textbox");
    textbox.setAttribute("id", "all-skill");
    textbox.setAttribute("value", "50");
    textbox.setAttribute("maxLength", "3");
    textbox.setAttribute("size", "3");
    textbox.setAttribute("onchange", "updateSkills()");
    row.appendChild(textbox);
    row.appendChild(document.createElement("spacer"));
    row.appendChild(document.createElement("spacer"));
    row.appendChild(document.createElement("spacer"));
    row.appendChild(document.createElement("spacer"));
    rows.appendChild(row);
    // Now add the weapon rows
    rows = document.getElementById("weapon-rows");
    row = null;
    for (var w in SkillChainDB.weapons) {
	var weapon = SkillChainDB.weapons[w];
	// create the checkbox for this weapon
	var checkbox = document.createElement("checkbox");
	checkbox.setAttribute("id", "weapon-" + w);
	checkbox.weapon = weapon;
	checkbox.setAttribute("label", weapon.name);
	if (row == null) {
	    row = document.createElement("row");
	    rows.appendChild(row);
	    row.appendChild(checkbox);
	} else {
	    row.appendChild(checkbox);
	    row = null;
	}
    }
    // Now add the avatar rows
    rows = document.getElementById("avatar-rows");
    for (var a in SkillChainDB.avatars) {
	var avatar = SkillChainDB.avatars[a];
	// create the checkbox for this avatar
	var checkbox = document.createElement("checkbox");
	checkbox.setAttribute("id", "avatar-" + a);
	checkbox.weapon = avatar;
	checkbox.setAttribute("label", avatar.name);
	rows.appendChild(checkbox);
    }
    var resultsFrame = document.getElementById("resultsFrame");
    resultsFrame.contentWindow.showSkillPopup = showSkillPopup;
    resultsFrame.contentWindow.hideSkillPopup = hideSkillPopup;
    resultsFrame.contentWindow.showChainPopup = showChainPopup;
    resultsFrame.contentWindow.hideChainPopup = hideChainPopup;
    if (getRememberSettings()) {
	loadSettings();
    }
    if (navigator.product == "Gecko" && navigator.productSub == "20050908") {
	var warn = document.createElement("description");
	warn.appendChild(document.createTextNode("WARNING: While the majority of this extension functions under Firefox 1.5 Beta 1, the tooltips in the result pane currently always pop up in the upper left hand corner.  This appears to be a Firefox bug relating to <iframe>s and tooltips, and not a bug within the extension.  While I am working on a work-around, for now, you'll have to live with it if you're using the beta version of Firefox."));
	document.documentElement.insertBefore(warn, document.getElementById("lookup-tabs"));
    }
}

function createMemberTextbox(rownum) {
    var textbox = document.createElement("textbox");
    textbox.setAttribute("id", "member" + rownum + "-label");
    textbox.setAttribute("value", getLocalString("member") + " " + rownum);
    textbox.setAttribute("disabled", "true");
    return textbox;
}

function createWeaponDropdown(rownum, nonelabel) {
    var dropdown = document.createElement("menulist");
    dropdown.setAttribute("id", "member" + rownum + "-weapon");
    dropdown.setAttribute("label", nonelabel);
    dropdown.setAttribute("oncommand", "changeWeaponOrJob('" + rownum + "')");
    var menu = document.createElement("menupopup");
    dropdown.appendChild(menu);
    var item = document.createElement("menuitem");
    item.setAttribute("label", nonelabel);
    item.setAttribute("selected", "true");
    menu.appendChild(item);
    for (var w in SkillChainDB.weapons) {
	item = document.createElement("menuitem");
	item.setAttribute("label", SkillChainDB.weapons[w].name);
	// set the "weapon" JavaScript property on this item
	item.weapon = SkillChainDB.weapons[w];
	menu.appendChild(item);
    }
    menu.appendChild(document.createElement("menuseparator"));
    item = document.createElement("menuitem");
    item.setAttribute("id", "member" + rownum + "-avatar-item");
    item.setAttribute("label", getLocalString("avatars"));
    item.avatar = true;
    menu.appendChild(item);
    item = document.createElement("menuitem");
    item.setAttribute("id", "member" + rownum + "-multiple-weapons-item");
    item.setAttribute("label", getLocalString("multiple.weapons"));
    item.multipleweps = true;
    menu.appendChild(item);
    return dropdown;
}

function createSkillBox(rownum) {
    var skillbox = document.createElement("skilllevel");
    skillbox.setAttribute("id", "member" + rownum + "-skill");
    skillbox.setAttribute("value", "50");
    skillbox.setAttribute("maxLength", "3");
    skillbox.setAttribute("size", "3");
    skillbox.setAttribute("disabled", "true");
    skillbox.setAttribute("onchange", "sanitizeSkillBox(this)");
    return skillbox;
}

function createJobDropdown(rownum, type, firstlabel) {
    var dropdown = document.createElement("menulist");
    dropdown.setAttribute("id", "member" + rownum + "-" + type);
    dropdown.setAttribute("label", firstlabel);
    dropdown.setAttribute("oncommand", "changeWeaponOrJob('" + rownum + "')");
    dropdown.setAttribute("disabled", "true");
    var menu = document.createElement("menupopup");
    dropdown.appendChild(menu);
    var item = document.createElement("menuitem");
    item.setAttribute("label", firstlabel);
    item.setAttribute("selected", "true");
    menu.appendChild(item);
    for (var j in SkillChainDB.jobs) {
	item = document.createElement("menuitem");
	item.setAttribute("label", SkillChainDB.jobs[j].name);
	// set the "job" JavaScript property on this item
	item.job = SkillChainDB.jobs[j];
	menu.appendChild(item);
    }
    return dropdown;
}

function createLabel(id) {
    var label = document.createElement("label");
    label.setAttribute("id", id);
    label.setAttribute("value", "");
    label.setAttribute("disabled", "true");
    return label;
}

function createColorDropdown(rownum) {
    var dropdown = document.createElement("menulist");
    dropdown.setAttribute("id", "member" + rownum + "-color");
    dropdown.setAttribute("label", colors[0].name);
    dropdown.setAttribute("disabled", "true");
    dropdown.setAttribute("oncommand", "colorChanged(this)");
    var menu = document.createElement("menupopup");
    dropdown.appendChild(menu);
    for (var c in colors) {
	item = document.createElement("menuitem");
	item.setAttribute("label", colors[c].name);
	if (colors[c].id != "custom" && colors[c].id != "normal") {
	    item.color = colors[c];
	    item.setAttribute("class", "color-" + colors[c].id);
	}
	menu.appendChild(item);
    }
    return dropdown;
}

function selectJob(job, jobDropdown) {
    for (var c = 0; c < jobDropdown.menupopup.childNodes.length; c++) {
	var jobObj = jobDropdown.menupopup.childNodes[c].job;
	if (jobObj != null && jobObj.id == job) {
	    jobDropdown.selectedIndex = c;
	    break;
	}
    }
}

// Load existing settings from the preferences if any are saved
// (settings are always saved)
function loadSettings() {
    var loadNames = getRememberNames();
    for (var i = 1; i <= MAX_PARTY_MEMBERS; i++) {
	if (loadNames) {
	    document.getElementById("member" + i + "-label").value = getSkillchainPref("member." + i + ".name", getLocalString("member") + " " + i);
	}
	document.getElementById("member" + i + "-skill").value = sanitizeSkill(parseInt(getSkillchainPref("member." + i + ".skill", "50")));
	selectJob(getSkillchainPref("member." + i + ".job", "none"), document.getElementById("member" + i + "-job"));
	selectJob(getSkillchainPref("member." + i + ".subjob", "none"), document.getElementById("member" + i + "-subjob"));
	var colorDropdown = document.getElementById("member" + i + "-color");
	var color = getSkillchainPref("member." + i + ".color", "normal");
	for (var c = 0; c < colorDropdown.menupopup.childNodes.length; c++) {
	    var colorObj = colorDropdown.menupopup.childNodes[c].color;
	    if (colorObj != null && colorObj.id == color) {
		colorDropdown.selectedIndex = c;
		break;
	    }
	}
	colorChanged(colorDropdown);
	var weaponDropdown = document.getElementById("member" + i + "-weapon");
	var weapon = getSkillchainPref("member." + i + ".weapon", "none");
	if (weapon == "none") {
	    weaponDropdown.selectedIndex = 0;
	    updateMemberRow(i);
	} else if (weapon.length > 8 && weapon.substring(0,8) == "avatars:") {
	    var avatars = weapon.substring(8).split(",");
	    var avatarArray = new Array();
	    // go through the avatars we know and select 'em
	    for (var a in avatars) {
		for (var dbA in SkillChainDB.avatars) {
		    if (avatars[a] == SkillChainDB.avatars[dbA].id) {
			avatarArray.push(SkillChainDB.avatars[dbA]);
			break;
		    }
		}
	    }
	    weaponDropdown.selectedItem = document.getElementById("member" + i + "-avatar-item");
	    // and activate them
	    avatarSelectCallback(i, avatarArray);
	} else if (weapon.length > 8 && weapon.substring(0,8) == "multwep:") {
	    var weapons = weapon.substring(8).split(",");
	    var weaponArray = new Array();
	    // go through the avatars we know and select 'em
	    for (var w in weapons) {
		for (var dbW in SkillChainDB.weapons) {
		    if (weapons[w] == SkillChainDB.weapons[dbW].id) {
			weaponArray.push(SkillChainDB.weapons[dbW]);
			break;
		    }
		}
	    }
	    weaponDropdown.selectedItem = document.getElementById("member" + i + "-multiple-weapons-item");
	    // and activate them
	    multWeaponSelectCallback(i, weaponArray);
	} else {
	    for (var c = 0; c < weaponDropdown.menupopup.childNodes.length; c++) {
		var weapObj = weaponDropdown.menupopup.childNodes[c].weapon;
		if (weapObj != null && weapObj.id == weapon) {
		    weaponDropdown.selectedIndex = c;
		    break;
		}
	    }
	    updateMemberRow(i);
	}
    }
    for (var w in SkillChainDB.weapons) {
	document.getElementById("weapon-" + w).checked = getSkillchainBoolPref("weapon." + w, false);
    }
    for (var a in SkillChainDB.avatars) {
	document.getElementById("avatar-" + a).checked = getSkillchainBoolPref("avatar." + a, false);
    }
    document.getElementById("weapon-chain-with-self").checked = getSkillchainBoolPref("selfchain", false);
    document.getElementById("weapon-skill-level").value = sanitizeSkill(parseInt(getSkillchainPref("weapons.skill", "50")));
    // now activate it possible
    checkTabs();
}

function saveSettings() {
    for (var i = 1; i <= MAX_PARTY_MEMBERS; i++) {
	setSkillchainPref("member." + i + ".name", document.getElementById("member" + i + "-label").value);
	setSkillchainPref("member." + i + ".skill", document.getElementById("member" + i + "-skill").value);
	var weaponDropdown = document.getElementById("member" + i + "-weapon");
	if (weaponDropdown.selectedItem.avatar) {
	    var str = null;
	    var avatars = weaponDropdown.selectedItem.avatars;
	    for (var a in avatars) {
		if (str == null) {
		    str = "avatars:";
		} else {
		    str += ",";
		}
		str += avatars[a].id;
	    }
	    setSkillchainPref("member." + i + ".weapon", str);
	} else if (weaponDropdown.selectedItem.multipleweps) {
	    var str = null;
	    var weapons = weaponDropdown.selectedItem.weapons;
	    for (var w in weapons) {
		if (str == null) {
		    str = "multwep:";
		} else {
		    str += ",";
		}
		str += weapons[w].id;
	    }
	    setSkillchainPref("member." + i + ".weapon", str);
	} else {
	    setSkillchainPref("member." + i + ".weapon", weaponDropdown.selectedItem.weapon == null ? "none" : weaponDropdown.selectedItem.weapon.id);
	    var job = document.getElementById("member" + i + "-job").selectedItem.job;
	    setSkillchainPref("member." + i + ".job", job == null ? "none" : job.id);
	    job = document.getElementById("member" + i + "-subjob").selectedItem.job;
	    setSkillchainPref("member." + i + ".subjob", job == null ? "none" : job.id);
	}
	var color = document.getElementById("member" + i + "-color").selectedItem.color;
	setSkillchainPref("member." + i + ".color", color == null ? "normal" : color.id);
    }
    for (var w in SkillChainDB.weapons) {
	setSkillchainBoolPref("weapon." + w, document.getElementById("weapon-" + w).checked);
    }
    for (var a in SkillChainDB.avatars) {
	setSkillchainBoolPref("avatar." + a, document.getElementById("avatar-" + a).checked);
    }
    // this is kind of complicated, since it can be auto-set but should
    // remember the old setting
    var chainWithSelfCheck = document.getElementById("weapon-chain-with-self");
    setSkillchainBoolPref("selfchain", chainWithSelfCheck.renew ? chainWithSelfCheck.lastChecked : chainWithSelfCheck.checked);
    setSkillchainPref("weapons.skill", document.getElementById("weapon-skill-level").value);
}

// Remove all current settings
function clearSettings() {
    if (confirm(getLocalString("confirm.clear.settings"))) {
	for (var i = 1; i <= MAX_PARTY_MEMBERS; i++) {
	    var dropdown = document.getElementById("member" + i + "-weapon");
	    dropdown.selectedIndex = 0;
	    dropdown.removeAttribute("tooltiptext");
	    dropdown = document.getElementById("member" + i + "-job");
	    dropdown.selectedIndex = 0;
	    dropdown.disabled = true;
	    dropdown = document.getElementById("member" + i + "-subjob");
	    dropdown.selectedIndex = 0;
	    dropdown.disabled = true;
	    dropdown = document.getElementById("member" + i + "-color");
	    dropdown.selectedIndex = 0;
	    dropdown.disabled = true;
	    dropdown.removeAttribute("class");
	    document.getElementById("member" + i + "-ex").value = "";
	    document.getElementById("member" + i + "-sp").value = "";
	    var skillField = document.getElementById("member" + i + "-skill");
	    skillField.value = "50";
	    skillField.disabled = true;
	    var label = document.getElementById("member" + i + "-label");
	    label.setAttribute("value", getLocalString("member") + " " + i);
	    label.setAttribute("disabled", "true");
	    skillField.disabled = true;
	}
	for (var w in SkillChainDB.weapons) {
	    document.getElementById("weapon-" + w).checked = false;
	}
	for (var a in SkillChainDB.avatars) {
	    document.getElementById("avatar-" + a).checked = false;
	}
	var checkbox = document.getElementById("weapon-chain-with-self");
	checkbox.checked = false;
	checkbox.disabled = true;
	document.getElementById("weapon-skill-level").value = "50";
	document.getElementById("go").disabled = disable;
    }
}

function sanitizeSkillBox(box) {
    var skill = parseInt(box.value);
    var nSkill = sanitizeSkill(skill);
    // prevent infinite looping by repeatedly changing the value to the same thing
    if (box.value != nSkill)
	box.value = nSkill;
}

function sanitizeSkill(skill) {
    if (skill >= 1 && skill <= 999) {
	// Valid
	return skill;
    } else {
	if (skill > 999) {
	    // there's no point in skills above 255 (yet), but the user can feel special
	    return 999;
	} else {
	    // this grabs NaN too (since all NaN comparisons are false)
	    return 1;
	}
    }
}

function updateSkills() {
    var box = document.getElementById("all-skill");
    var skill = parseInt(box.value);
    var nSkill = sanitizeSkill(skill);
    // prevent infinite looping by repeatedly changing the value to the same thing
    if (box.value != nSkill)
	box.value = nSkill;
    // set all the other skills
    for (var i = 1; i <= MAX_PARTY_MEMBERS; i++) {
	document.getElementById("member" + i + "-skill").value = nSkill;
    }
}

function checkTabs() {
    var lookupType = document.getElementById("lookup-tabs").selectedIndex;
    if (lookupType == 0) {
	checkMembers();
    } else if (lookupType == 1) {
	checkWeapons();
    }
}

function checkMembers() {
    // count members available, disable "calculate" if none
    var disable = true;
    var count = 0;
    for (var i = 1; i <= MAX_PARTY_MEMBERS; i++) {
	var item = document.getElementById("member" + i + "-weapon").selectedItem;
	if (item.weapon != null || item.avatar) {
	    count++;
	    if (count >= 2) {
		disable = false;
		break;
	    }
	}
    }
    document.getElementById("go").disabled = disable;
}

function colorChanged(colorDropdown) {
    var color = colorDropdown.selectedItem.color;
    if (color == null || color.id == "normal") {
	colorDropdown.removeAttribute("class");
    } else if (color.id == "custom") {
	// choose a custom color
	alert("Custom colors are not yet operational.");
    } else {
	colorDropdown.setAttribute("class", "color-" + color.id);
    }
}

function avatarSelectCallback(rownum, avatars) {
    var disable = false;
    var exlabel = document.getElementById("member" + rownum + "-ex");
    var splabel = document.getElementById("member" + rownum + "-sp");
    if (avatars == null || avatars.length == 0) {
	var weaponDropdown = document.getElementById("member" + rownum + "-weapon");
	document.getElementById("member" + rownum + "-weapon").selectedIndex = 0;
	weaponDropdown.removeAttribute("tooltiptext");
	disable = true;
	exlabel.setAttribute("value", "");
	splabel.setAttribute("value", "");
    } else {
	// Update the menu item
	var avatarItem = document.getElementById("member" + rownum + "-avatar-item");
	avatarItem.avatars = avatars;
	var label = null;
	for (var i in avatars) {
	    if (label == null) {
		label = avatars[i].name;
	    } else {
		label += ", " + avatars[i].name;
	    }
	}
	var weaponDropdown = document.getElementById("member" + rownum + "-weapon");
	weaponDropdown.setAttribute("label", avatars.length == SkillChainDB.avatars.length ? getLocalString("avatar.all") : label);
	weaponDropdown.setAttribute("tooltiptext", label);
	exlabel.setAttribute("value", "--");
	splabel.setAttribute("value", "--");
    }
    document.getElementById("member" + rownum + "-label").disabled = disable;
    document.getElementById("member" + rownum + "-skill").disabled = disable;
    document.getElementById("member" + rownum + "-color").disabled = disable;
    document.getElementById("member" + rownum + "-job").disabled = true;
    document.getElementById("member" + rownum + "-subjob").disabled = true;
    exlabel.disabled = true;
    splabel.disabled = true;
}

function multWeaponSelectCallback(rownum, weapons) {
    var disable = false;
    var exlabel = document.getElementById("member" + rownum + "-ex");
    var splabel = document.getElementById("member" + rownum + "-sp");
    if (weapons == null || weapons.length == 0) {
	var weaponDropdown = document.getElementById("member" + rownum + "-weapon");
	document.getElementById("member" + rownum + "-weapon").selectedIndex = 0;
	weaponDropdown.removeAttribute("tooltiptext");
	disable = true;
	exlabel.setAttribute("value", "");
	splabel.setAttribute("value", "");
    } else {
	// Update the menu item
	var multWepsItem = document.getElementById("member" + rownum + "-multiple-weapons-item");
	multWepsItem.weapons = weapons;
	var label = null;
	for (var i in weapons) {
	    if (label == null) {
		label = weapons[i].name;
	    } else {
		label += ", " + weapons[i].name;
	    }
	}
	var weaponDropdown = document.getElementById("member" + rownum + "-weapon");
	weaponDropdown.setAttribute("label", label);
	weaponDropdown.setAttribute("tooltiptext", label);
	exlabel.setAttribute("value", "--");
	splabel.setAttribute("value", "--");
    }
    document.getElementById("member" + rownum + "-label").disabled = disable;
    document.getElementById("member" + rownum + "-skill").disabled = disable;
    document.getElementById("member" + rownum + "-color").disabled = disable;
    document.getElementById("member" + rownum + "-job").disabled = disable;
    document.getElementById("member" + rownum + "-subjob").disabled = disable;
    exlabel.disabled = true;
    splabel.disabled = true;
}

function changeWeaponOrJob(rownum) {
    // Figure out what weapon and job this has been set to
    var weaponDropdown = document.getElementById("member" + rownum + "-weapon");
    if (weaponDropdown.selectedItem.avatar) {
	// we're selecting a group of avatars, show the avatar dialog
	document.getElementById("member" + rownum + "-skill").weapon = null;
	window.openDialog("chrome://skillchains/content/skillchains-avatars.xul", "avatars", "chrome,modal,x=" + weaponDropdown.boxObject.screenX + ",y=" + weaponDropdown.boxObject.screenY, rownum, weaponDropdown.selectedItem.avatars);
	return;
    } else if (weaponDropdown.selectedItem.multipleweps) {
	// we're selecting a group of weapons, show the multiple weapons dialog
	document.getElementById("member" + rownum + "-skill").weapon = null;
	window.openDialog("chrome://skillchains/content/skillchains-multiple-weps.xul", "avatars", "chrome,modal,x=" + weaponDropdown.boxObject.screenX + ",y=" + weaponDropdown.boxObject.screenY, rownum, weaponDropdown.selectedItem.avatars);
	return;
    }
    updateMemberRow(rownum);
}

function updateMemberRow(rownum) {
    var weaponDropdown = document.getElementById("member" + rownum + "-weapon");
    weaponDropdown.removeAttribute("tooltiptext");
    var weapon = weaponDropdown.selectedItem.weapon;
    var jobDropdown = document.getElementById("member" + rownum + "-job");
    var job = jobDropdown.selectedItem.job;
    var subjobDropdown = document.getElementById("member" + rownum + "-subjob");
    var subjob = subjobDropdown.selectedItem.job;
    var exlabel = document.getElementById("member" + rownum + "-ex");
    var splabel = document.getElementById("member" + rownum + "-sp");
    var ex = false, sp = false;
    var disable = weapon == null;
    var skillField = document.getElementById("member" + rownum + "-skill");
    document.getElementById("member" + rownum + "-label").disabled = disable;
    document.getElementById("member" + rownum + "-color").disabled = disable;
    skillField.disabled = disable;
    exlabel.disabled = disable;
    splabel.disabled = disable;
    jobDropdown.disabled = disable;
    subjobDropdown.disabled = disable;
    skillField.weapon = weapon;
    if (weapon != null && weapon.avatar) {
	// don't bother with the rest, it doesn't mean anything
	return;
    }
    if (weapon != null && job != null) {
	// find out if this job is EX or SP
	for (j in weapon.ex) {
	    if (job == weapon.ex[j] || subjob == weapon.ex[j]) {
		ex = true;
		break;
	    }
	}
	for (j in weapon.sp) {
	    if (job == weapon.sp[j]) {
		sp = true;
		break;
	    }
	}
	exlabel.setAttribute("value", ex ? getLocalString("ex") : "");
	splabel.setAttribute("value", sp ? getLocalString("sp") : "");
    } else {
	if (weapon == null) {
	    // no weapon, no ex/sp
	    exlabel.setAttribute("value", "");
	    splabel.setAttribute("value", "");
	} else {
	    // no job selected, use both
	    exlabel.setAttribute("value", getLocalString("ex"));
	    splabel.setAttribute("value", getLocalString("sp"));
	}
    }
}

function checkWeapons() {
    var count = 0;
    for (var w in SkillChainDB.weapons) {
	if (document.getElementById("weapon-" + w).checked) {
	    count++;
	    if (count >= 2)
		break;
	}
    }
    for (var a in SkillChainDB.avatars) {
	if (document.getElementById("avatar-" + a).checked) {
	    count++;
	    if (count >= 2)
		break;
	}
    }
    var chainWithSelfCheck = document.getElementById("weapon-chain-with-self");
    var calculateButton = document.getElementById("go");
    if (count == 0) {
	// disable everything
	chainWithSelfCheck.disabled = true;
	calculateButton.disabled = true;
    } else if (count == 1) {
	// force the "chain with own" checkbox to be checked and check it
	if (!chainWithSelfCheck.renew) {
	    chainWithSelfCheck.renew = true;
	    chainWithSelfCheck.lastChecked = chainWithSelfCheck.checked;
	    chainWithSelfCheck.checked = true;
	}
	chainWithSelfCheck.disabled = true;
	// allow calculations
	calculateButton.disabled = false;
    } else {
	// if the value needs to be restored, restore it
	if (chainWithSelfCheck.renew) {
	    chainWithSelfCheck.renew = false;
	    chainWithSelfCheck.checked = chainWithSelfCheck.lastChecked;
	}
	chainWithSelfCheck.disabled = false;
	calculateButton.disabled = false;
    }
}

function toggleOptions() {
    var options = document.documentElement.getAttribute("options");
    var cssClass;
    if (options == "hide") {
	document.documentElement.setAttribute("options", "show");
	cssClass = "up";
    } else {
	document.documentElement.setAttribute("options", "hide");
	cssClass = "down";
    }
    document.getElementById("results-options").setAttribute("class", cssClass);
}

function toggleLookup() {
    var lookup = document.documentElement.getAttribute("lookup");
    var cssClass;
    if (lookup == "hide") {
	document.documentElement.setAttribute("lookup", "show");
	cssClass = "up";
    } else {
	document.documentElement.setAttribute("lookup", "hide");
	cssClass = "down";
    }
    document.getElementById("toggle-lookup").setAttribute("class", cssClass);
}

//
// Calculates that chains and shows the results.
//
function calculateChains() {
    // disable some elements that need disabling
    document.getElementById("go").disabled = true;
    document.getElementById("lookup-tabs").disabled = true;
    // switch to the progress meter card
    document.getElementById("resultsDeck").setAttribute("selectedIndex", "1");
    // set progress to 0
    currentProgressCount = 0;
    totalProgressCount = 1;
    document.getElementById("resultsMeter").setAttribute("value", "0%");
    // Moz seems to try and be "smart" about event handlers and doesn't
    // redraw the UI until after the thing has updated, so "fork" into
    // another thread (kinda)
    progressInterval = setInterval(checkProgress, 500);
    setTimeout(runChains, 10);
}

function checkProgress() {
    document.getElementById("resultsMeter").setAttribute("value", (currentProgressCount * 100 / totalProgressCount) + "%");
}

function runChains() {
    try {
	var deck = document.getElementById("resultsDeck");
	var lookupType = document.getElementById("lookup-tabs").selectedIndex;
	var hideImpossibleChains = true;
	var enablePlayerNames = true;
	// kill the print/options buttons now
	document.getElementById("cmdPrintResults").setAttribute("disabled", "true");
	document.getElementById("results-options").disabled = true;
	document.getElementById("toggle-lookup").disabled = true;
	// start actually calculating
	var chains = new Array();
	if (lookupType == 0) {
	    // create a list of players we're chaining with
	    var players = new Array();
	    //var debug = "The following players were created:";
	    for (var i = 1; i <= MAX_PARTY_MEMBERS; i++) {
		var weaponDropdown = document.getElementById("member" + i + "-weapon");
		var player = null;
		var job = document.getElementById("member" + i + "-job").selectedItem.job;
		var subjob = document.getElementById("member" + i + "-subjob").selectedItem.job;
		var playerName = document.getElementById("member" + i + "-label").value;
		var skill = sanitizeSkill(parseInt(document.getElementById("member" + i + "-skill").value));
		if (weaponDropdown.selectedItem.avatar) {
		    // create an instance of the player for each avatar
		    player = new Player(weaponDropdown.selectedItem.avatars, skill, null, null, playerName);
		} else if (weaponDropdown.selectedItem.multipleweps) {
		    player = new Player(weaponDropdown.selectedItem.weapons, skill, job, subjob, playerName);
		} else {
		    var weapon = weaponDropdown.selectedItem.weapon;
		    if (weapon != null) {
			// there's data here
			/*debug += "\r\n - Player " + i + " uses " + weapon.name + " at skill " + skill;*/
			// add the player
			player = new Player(weapon, skill, job, subjob, playerName);
		    }
		}
		if (player != null) {
		    // one of the fun things about JavaScript is we can add custom properties to items
		    var color = document.getElementById("member" + i + "-color").selectedItem.color;
		    if (color != null) {
			player.color = "#" + color.foreground;
			player.backgroundColor = "#" + color.background;
		    }
		    players.push(player);
		}
	    }
	    //alert(debug);
	    if (players.length < 2) {
		alert(getLocalString("too.few.players"));
		// switch back to the help card
		deck.setAttribute("selectedIndex", "0");
		return;
	    }
	    // start calculating, based on the number of players
	    totalProgressCount = players.length * (players.length - 1);
	    chains = calculatePlayerChains(players);
	} else if (lookupType == 1) {
	    // add the weapons and look up through them
	    var weapons = new Array();
	    for (var w in SkillChainDB.weapons) {
		var checkbox = document.getElementById("weapon-" + w);
		if (checkbox.checked) {
		    weapons.push(checkbox.weapon);
		}
	    }
	    for (var a in SkillChainDB.avatars) {
		var checkbox = document.getElementById("avatar-" + a);
		//dump("Checking " + SkillChainDB.avatars[a].name + " (box " + checkbox + ")\n");
		if (checkbox.checked) {
		    weapons.push(checkbox.weapon);
		}
	    }
	    var withSelf = document.getElementById("weapon-chain-with-self").checked;
	    if (weapons.length < (withSelf ? 1 : 2)) {
		alert(getLocalString("too.few.weapons." + (withSelf ? "own" : "individual")));
		// switch back to the help card
		deck.setAttribute("selectedIndex", "0");
		return;
	    }
	    var skill = sanitizeSkill(parseInt(document.getElementById("weapon-skill-level").value));
	    // start calculating
	    currentProgressCount = 0;
	    totalProgressCount = weapons.length * (weapons.length - (withSelf ? 0 : 1));
	    var i, j;
	    for (i in weapons) {
		for (j in weapons) {
		    // can't chain with ourselves
		    if (withSelf || i != j) {
			chains = chains.concat(weapons[i].findChains(weapons[j], skill));
			currentProgressCount++;
		    }
		}
	    }
	    // go ahead and show impossible chains (ie, light and dark)
	    hideImpossibleChains = false;
	    enablePlayerNames = false;
	} else {
	    alert("This tab (" + lookupType + ") isn't implemented yet.  As far as I know, it doesn't exist.");
	    // switch back to the help card
	    deck.setAttribute("selectedIndex", "0");
	    return;
	}
	// we find anything?
	if (chains.length > 0) {
	    chains.sort(skillChainCompare);
	    var useNewResults = false;
	    var arrowType = getArrowType();
	    var showPlayerNamesCheck = document.getElementById("showPlayerNames");
	    showPlayerNamesCheck.style.display = enablePlayerNames ? "-moz-box" : "none";
	    document.getElementById("resultsFrame").contentDocument.body.setAttribute("players", enablePlayerNames && showPlayerNamesCheck.checked ? "show" : "hide");
	    createHTMLTree(chains, arrowType);
	    applyResultsOptions();
	    // ok - now switch to the results part of the deck
	    deck.setAttribute("selectedIndex", "2");
	    // restore the print/options button now too
	    document.getElementById("cmdPrintResults").removeAttribute("disabled");
	    document.getElementById("results-options").disabled = false;
	    // allow them to hide the lookup options now
	    document.getElementById("toggle-lookup").disabled = false;
	} else {
	    // nope - show the error part of the deck
	    deck.setAttribute("selectedIndex", "3");
	}
    } catch (ex) {
	alert ("An error occured while looking up chains:\r\n\r\n" + dumpError(ex));
    } // finally, reenable things
    document.getElementById("go").disabled = false;
    document.getElementById("lookup-tabs").disabled = false;
    if (progressInterval != null) {
	clearInterval(progressInterval);
	progressInterval = null;
    }
}

function dumpError(ex) {
    if (ex.name && ex.fileName && ex.lineNumber && ex.message) {
	var msg = ex.name + ": " + ex.message + " in " + ex.fileName + ", line " + ex.lineNumber;
	dump(msg);
	dump("\n");
	for (var i in ex.stack) {
	    dump(i + ": " + ex.stack[i] + "\n");
	}
	return msg;
    }
    return ex;
}

// Builds the HTML results.  The HTML results contain additional information
// and better mouseover support, but don't use the native tree widget.
//
// It also has the added advantage that it supports printing.
function createHTMLTree(chains, arrowType) {
    var doc = document.getElementById("resultsFrame").contentDocument;
    var div = doc.getElementById("resultsDiv");
    // remove any contents from this div
    var i;
    for (i = div.childNodes.length; i > 0; i--) {
	div.removeChild(div.childNodes.item(i-1));
    }
    // Create the tree
    createHTMLChainTree(doc, div, chains, arrowType);
}

function createHTMLChainIcon(doc, parent, chain) {
    if (chain == null)
	return;
    var img = doc.createElement("img");
    img.src = "chrome://skillchains/skin/sc/" + chain.icon + ".png";
    img.setAttribute("alt", chain.name);
    img.setAttribute("class", "chainIcon");
    img.style.width = "16px";
    img.style.height = "16px";
    img.style.verticalAlign = "middle";
    parent.appendChild(img);
}


function createHTMLElementIcon(doc, parent, element) {
    if (element == null)
	return;
    var img = doc.createElement("img");
    img.src = "chrome://skillchains/skin/elements/" + element.icon + ".png";
    img.setAttribute("alt", element.name);
    img.style.width = "16px";
    img.style.height = "16px";
    img.style.verticalAlign = "middle";
    parent.appendChild(img);
}

function createHTMLElementIcons(doc, parent, skill) {
    createHTMLElementIcon(doc, parent, skill.element);
    createHTMLElementIcon(doc, parent, skill.element2);
    createHTMLElementIcon(doc, parent, skill.element3);
}

function createHTMLChainElementIcons(doc, parent, chain) {
    createHTMLElementIcon(doc, parent, chain.chainElement1);
    createHTMLElementIcon(doc, parent, chain.chainElement2);
    createHTMLElementIcon(doc, parent, chain.chainElement3);
    createHTMLElementIcon(doc, parent, chain.chainElement4);
}

function createHTMLSkill(doc, parent, skill, player) {
    var skillSpan = doc.createElement("span");
    skillSpan.setAttribute("class", "skill");
    skillSpan.skill = skill;
    skillSpan.player = player;
    if (player != null && player.color != null && player.backgroundColor != null) {
	skillSpan.style.color = player.color;
	skillSpan.style.backgroundColor = player.backgroundColor;
    }
    var span = doc.createElement("span");
    span.setAttribute("class", "skillElement");
    createHTMLElementIcons(doc, span, skill);
    skillSpan.appendChild(span);
    skillSpan.appendChild(doc.createTextNode(skill.name));
    span = doc.createElement("span");
    span.setAttribute("class", "playerName");
    span.appendChild(doc.createTextNode(" [" + (player == null ? "unknown" : player.name) + "]"));
    skillSpan.appendChild(span);
    span = doc.createElement("span");
    span.setAttribute("class", "weaponName");
    span.appendChild(doc.createTextNode(" [" + skill.weapon.name + "]"));
    skillSpan.appendChild(span);
    parent.appendChild(skillSpan);
}
// Creates the DOM structure to display an individual skill.
function createHTMLSkillChain(doc, parent, chain, arrowType) {
    createHTMLSkill(doc, parent, chain.fromSkill, chain.fromPlayer);
    if (arrowType == "unicode") {
	parent.appendChild(doc.createTextNode(" \u27A8 "));
    } else {
	arrow = doc.createElement("img");
	arrow.src = "chrome://skillchains/skin/icons/arrow.png";
	arrow.setAttribute("class", "arrow-image");
	parent.appendChild(arrow);
    }
    createHTMLSkill(doc, parent, chain.toSkill, chain.toPlayer);
}

function createTwisty(doc, parent, id) {
    // add a expander/collapser
    var twisty = doc.createElement("img");
    twisty.setAttribute("class", "results-twisty");
    twisty.src = "chrome://skillchains/skin/icons/collapse.png";
    twisty.setAttribute("onclick", "toggle(this, " + id + ")");
    parent.appendChild(twisty);
}

function createHTMLChainTree(doc, parent, chains, arrowType) {
    var prevChainType = null;
    var prevChain = null;
    var container = null;
    var block, arrow;
    for (i in chains) {
	if (prevChain != null) {
	    if (prevChain.equals(chains[i]))
		continue;
	}
	prevChain = chains[i];
	// still in the same header?
	if (chains[i].chain != prevChainType) {
	    // create a header
	    var header = doc.createElement("div");
	    var id = elementGUIDCounter++;
	    header.setAttribute("class", "results-header");
	    header.setAttribute("id", "h" + id);
	    createTwisty(doc, header, id);
	    var chainSpan = doc.createElement("span");
	    chainSpan.setAttribute("class", "chain");
	    chainSpan.chain = chains[i].chain;
	    createHTMLChainIcon(doc, chainSpan, chains[i].chain);
	    var span = doc.createElement("span");
	    span.setAttribute("class", "chainElement");
	    createHTMLChainElementIcons(doc, span, chains[i].chain);
	    chainSpan.appendChild(span);
	    chainSpan.appendChild(doc.createTextNode(chains[i].chain.name));
	    var burstLabel = " \u2013 " + chains[i].chain.chainElement1.name;
	    if (chains[i].chain.chainElement2 != null) {
		burstLabel += ", " + chains[i].chain.chainElement2.name;
		if (chains[i].chain.chainElement3 != null) {
		    burstLabel += ", " + chains[i].chain.chainElement3.name;
		    if (chains[i].chain.chainElement4 != null) {
			burstLabel += ", " + chains[i].chain.chainElement4.name;
		    }
		}
	    }
	    span = doc.createElement("span");
	    span.setAttribute("class", "chainElementNames");
	    span.appendChild(document.createTextNode(burstLabel));
	    chainSpan.appendChild(span);
	    header.appendChild(chainSpan);
	    parent.appendChild(header);
	    container = doc.createElement("div");
	    container.setAttribute("class", "results-tree-children");
	    container.setAttribute("id", "c" + id);
	    header.appendChild(container);
	    prevChainType = chains[i].chain;
	}
	var myId = elementGUIDCounter++;
	block = doc.createElement("div");
	if (chains[i].nextChain.length > 0) {
	    createTwisty(doc, block, myId);
	} else {
	    // create a spacer to take the place of the twisty
	    var spacer = doc.createElement("img");
	    spacer.setAttribute("class", "results-twisty-spacer");
	    spacer.src = "chrome://skillchains/skin/icons/none.png";
	    block.appendChild(spacer);
	}
	block.setAttribute("id", "sc" + myId);
	createHTMLSkillChain(doc, block, chains[i], arrowType);
	container.appendChild(block);
	if (chains[i].nextChain.length > 0) {
	    var children = document.createElement("div");
	    children.setAttribute("id", "c" + myId);
	    children.setAttribute("class", "results-tree-children");
	    block.appendChild(children);
	    chains[i].nextChain.sort(skillChainCompare);
	    createHTMLChainTree(doc, children, chains[i].nextChain, arrowType);
	}
    }
}

function toggleWeaponNames() {
    var show = document.getElementById("showWeaponNames").checked;
    document.getElementById("resultsFrame").contentDocument.body.setAttribute("weapons", show ? "show" : "hide");
}

function togglePlayerNames() {
    var show = document.getElementById("showPlayerNames").checked;
    document.getElementById("resultsFrame").contentDocument.body.setAttribute("players", show ? "show" : "hide");
}

/* Applies the options to the body element of the results display */
function applyResultsOptions() {
    var body = document.getElementById("resultsFrame").contentDocument.body;
    body.setAttribute("skillelements", getShowElements() ? "show" : "hide");
    body.setAttribute("chainicons", getShowChainIcons() ? "show" : "hide");
    body.setAttribute("chainelements", getShowChainElements() ? "show" : "hide");
    body.setAttribute("chainelementnames", getShowChainElementNames() ? "show" : "hide");
}

function printPreviewResults() {
    alert("Not implemented yet.");
    //document.getElementById("resultsFrame").contentWindow.PrintUtils.printPreview();
}

function printResults() {
    document.getElementById("resultsFrame").contentWindow.print();
}

function showSkillPopup(elem, x, y) {
    // Only show the tooltip if the given element has an attached
    // weapon skill to display information for
    if (elem.skill == null)
	return;
    // Update the popup with the skill information.
    var popup = document.getElementById("weaponSkillTip");
    document.getElementById("weaponSkillTipElement1").setAttribute("element", elem.skill.element == null ? "none" : elem.skill.element.icon);
    document.getElementById("weaponSkillTipElement2").setAttribute("element", elem.skill.element2 == null ? "none" : elem.skill.element2.icon);
    document.getElementById("weaponSkillTipElement3").setAttribute("element", elem.skill.element3 == null ? "none" : elem.skill.element3.icon);
    document.getElementById("weaponSkillTipName").setAttribute("value", elem.skill.name);
    document.getElementById("weaponSkillTipType").setAttribute("value",
	getLocalString("tooltip.skill.level") + " " + elem.skill.level + " " +
	    elem.skill.weapon.name + " " +
		((elem.skill.flags & WS_AVATAR) ?
		    getLocalString("tooltip.skill.pact") :
		    getLocalString("tooltip.skill.weapon")));
    var label = document.getElementById("weaponSkillTipExSp");
    if (elem.skill.flags == WS_EX) {
	var msg = getLocalString("tooltip.ex");
	var comma = false;
	for (var i in elem.skill.weapon.ex) {
	    if (comma) {
		msg += ", ";
	    } else {
		msg += " ";
		comma = true;
	    }
	    msg += elem.skill.weapon.ex[i].name;
	}
	label.setAttribute("value", msg);
	label.style.display = "-moz-box";
    } else if (elem.skill.flags == WS_SP) {
	var msg = getLocalString("tooltip.sp");
	var comma = false;
	for (var i in elem.skill.weapon.sp) {
	    if (comma) {
		msg += ", ";
	    } else {
		msg += " ";
		comma = true;
	    }
	    msg += elem.skill.weapon.sp[i].name;
	}
	label.setAttribute("value", msg);
	label.style.display = "-moz-box";
    } else {
	label.style.display = "none";
    }
    label = document.getElementById("weaponSkillTipPlayer");
    if (elem.player == null) {
	label.style.display = "none";
    } else {
	label.style.display = "-moz-box";
	label.setAttribute("value", elem.player.name);
    }
    x += window.screenX + (window.outerWidth - window.innerWidth);
    y += window.screenY + (window.outerHeight - window.innerHeight);
    //dump("popup over " + elem + " @ " + x + ", " + y + "\n");
    popup.moveTo(x, y);
    popup.showPopup(elem, x, y, "tooltip");
}

function hideSkillPopup() {
    document.getElementById("weaponSkillTip").hidePopup();
}

function showChainPopup(elem, x, y) {
    // Only show the tooltip if the given element has an attached
    // weapon skill to display information for
    if (elem.chain == null)
	return;
    // Update the popup with the skill information.
    var popup = document.getElementById("skillChainTip");
    document.getElementById("skillChainTipElement1").setAttribute("element", elem.chain.chainElement1 == null ? "none" : elem.chain.chainElement1.icon);
    document.getElementById("skillChainTipElement2").setAttribute("element", elem.chain.chainElement2 == null ? "none" : elem.chain.chainElement2.icon);
    document.getElementById("skillChainTipElement3").setAttribute("element", elem.chain.chainElement3 == null ? "none" : elem.chain.chainElement3.icon);
    document.getElementById("skillChainTipElement4").setAttribute("element", elem.chain.chainElement4 == null ? "none" : elem.chain.chainElement4.icon);
    document.getElementById("skillChainTipName").setAttribute("value", elem.chain.name);
    var burstLabel = elem.chain.chainElement1.name;
    if (elem.chain.chainElement2 != null) {
	burstLabel += ", " + elem.chain.chainElement2.name;
	if (elem.chain.chainElement3 != null) {
	    burstLabel += ", " + elem.chain.chainElement3.name;
	    if (elem.chain.chainElement4 != null) {
		burstLabel += ", " + elem.chain.chainElement4.name;
	    }
	}
    }
    document.getElementById("skillChainTipBurstWith").setAttribute("value", burstLabel);
    x += window.screenX + (window.outerWidth - window.innerWidth);
    y += window.screenY + (window.outerHeight - window.innerHeight);
    //dump("popup over " + elem + " @ " + x + ", " + y + "\n");
    popup.correctX = x;
    popup.correctY = y;
    popup.showPopup(elem, x, y, "tooltip");
    popup.moveTo(x, y);
}

function hideChainPopup() {
    document.getElementById("skillChainTip").hidePopup();
}
